package lk.sliit.research.interviewapp.controller;

import lk.sliit.research.interviewapp.Service.CandidateService;
import lk.sliit.research.interviewapp.model.domain.Candidate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import static lk.sliit.research.interviewapp.util.Constant.APP_NAME;
import static lk.sliit.research.interviewapp.util.Constant.ERROR_LOG;

@Slf4j
@RestController
@RequestMapping("candidate")
public class CandidateController {

    @Autowired
    CandidateService candidateService;

    @GetMapping("addCandidateView/{jobId}")
    public ModelAndView addCandidateView(@PathVariable int jobId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.setViewName("candidate/addCandidateView");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("jobId", jobId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

    @PostMapping("createCandidate")
    public boolean createCandidate(@RequestBody Candidate candidate){
        log.info("Request: {}", candidate);

        try{
            return candidateService.createCandidate(candidate);
        }
        catch (Exception e){
            log.error(ERROR_LOG, e);
            return false;
        }
    }

    @GetMapping("viewAllCandidates/{jobId}")
    public ModelAndView viewAllCandidates(@PathVariable int jobId){
        ModelAndView modelAndView = new ModelAndView();

        try{
            modelAndView.addObject("candidateList", candidateService.viewAllCandidatesByJobId(jobId));
            modelAndView.setViewName("candidate/listAllCandidates");
            modelAndView.addObject("text", APP_NAME);
            modelAndView.addObject("jobId", jobId);
        }
        catch (Exception e){
            modelAndView.setViewName("error/page");
            modelAndView.addObject("error", e.getMessage());
            log.error(ERROR_LOG, e);
        }
        return modelAndView;
    }

}
